package com.sundata.internalevaluation.calc.calcUnit;

import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.sundata.admin.AdminUtil;
import com.sundata.common.util.DBExecutor;
import com.sundata.common.util.JsonUtil;
import com.sundata.internalevaluation.calc.calcUnit.interfaces.InterfaceRunning;
import com.sundata.internalevaluation.calc.model.CalcException;
import com.sundata.internalevaluation.calc.model.CalcResult;
import com.sundata.internalevaluation.calc.model.CalcUnit;
import com.sundata.internalevaluation.calc.model.finals.CalcType;
import com.sundata.internalevaluation.calc.util.InvokeUtil;
import com.sundata.internalevaluation.configuration.model.SysInterface;
import com.sundata.internalevaluation.configuration.model.SysInterfaceParam;
import com.sundata.internalevaluation.script.TemplateUtil;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import org.apache.commons.text.StringSubstitutor;
import org.apache.flink.connector.jdbc.utils.JdbcUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.swing.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 *
 * @author JoeLazy
 * @date 2025-02-12 10:40:49
 * @description: 接口计算单元
 */
public class InterfaceCalcUnit extends CalcUnit {

    private static final Logger log = LoggerFactory.getLogger(InterfaceCalcUnit.class);


    private final SysInterface sysInterface;


    /**
     * 创建数据单元的绝对对象，对象必须包含如下参数
     *
     * @param calcCode    计算对象编号
     * @param calcName    计算对象名称
     * @param calcType    计算类型
     * @param initContext 计算单元初始化参数
     */
    public InterfaceCalcUnit(String calcCode, String calcName, CalcType calcType, Map<String, Object> initContext, SysInterface sysInterface) {
        super(calcCode, calcName, calcType, initContext);
        this.sysInterface = sysInterface;
    }

    /**
     * 判断是否已经计算过数据了
     *
     * @param calculateInstanceNumber 计算流水号
     * @return 是否计算过 true 计算过 false 没有计算过
     */
    @Override
    public boolean isCalcFinished(String calculateInstanceNumber) {
        return false;
    }

    /**
     * 初始化计算结果的方法，如果已经计算过，在实现过程中，应当在此方法中根据计算流水号重新初始化 resultContext 结果对象，为其他依赖对象做准备
     * 若明明计算过本单元但再次计算时没有初始化该对象，则计算依赖出现问题无法定位与处理
     *
     * @param calculateInstanceNumber 计算流水号
     */
    @Override
    public void initResultContext(String calculateInstanceNumber) {

    }

    /**
     * 根据节点配置获取源节点；
     *
     * @return 所有源头节点
     */
    @Override
    public List<CalcUnit> getSourceCalcUnits() {
//        List<SysInterfaceParam> paramList = sysInterface.getParamList();
//        List<CalcUnit> childCalcUnitList = paramList.stream()
//                .map(item -> new InterfaceParamCalcUnit(item.getParamObjName(), item.getParamChineseName(), CalcType.INTERFACEPARAM, Map.of(), item))
//                .collect(Collectors.toList());
//        return childCalcUnitList;

        return List.of();
    }

    /**
     * 计算之后的方法，可实现为空
     *
     * @param context
     */
    @Override
    public void afterCalc(Map<String, Object> context) {

    }

    /**
     * 计算之前，可实现空
     *
     * @param context
     */
    @Override
    public void beforeCalc(Map<String, Object> context) {

    }

    /**
     * 必须实现的主体计算内容
     *
     * @param thisResult              本计算单元的结果
     * @param calculateInstanceNumber 计算流水号
     * @param context                 节点计算参数清单
     * @param sourceResults           源头计算节点的结果
     */
    @Override
    public void calc(CalcResult<String, Object> thisResult,
                     String calculateInstanceNumber,
                     Map<String, Object> context,
                     Map<CalcUnit, CalcResult<String, Object>> sourceResults) {

        log.debug("计算{}：编号:{}，名称:{}，流水号:{}", getCalcType().getName(), getCalcCode(), getCalcName(), calculateInstanceNumber);
        // 获取新流水号，每次请求接口必须不一样
        String globalReqNo = AdminUtil.getNextId("cretSeqno");
        context.put("globalReqNo",globalReqNo);
        // creditSeqNo 查询正i性能的流水号 如果不存在就重新赋值 只有征信的请求才会用到
        context.putIfAbsent("creditSeqNo",globalReqNo);

        // 接口内容类型
        String reqContentTypeCode = this.sysInterface.getReqContentTypeCode();
        // 接口请求类型
        String interfaceTypeCode = this.sysInterface.getInterfaceTypeCode();

        // 请求类型不为 http或者 https   或者   请求体不为 json
        if (!interfaceTypeCode.startsWith("http") || !"json".equals(reqContentTypeCode)) {
            // TODO Socket 请求
            throw new CalcException("目前仅支持http和https的请求和json请求体");
        }

        // 请求参数实体列表
        List<SysInterfaceParam> paramList = sysInterface.getParamList();
        Map<String, String> headersMap = new HashMap<>();
        Map<String, Object> paramMap = new HashMap<>();

        // 封装请求参数
        paramList.forEach(
                param -> {
                    Object result = getParamValue(param, context);
                    String isHeader = param.getIsHeader();
                    if ("true".equals(isHeader)) {
                        // 请求头信息
                        headersMap.put(param.getParamObjName(), JsonUtil.toJSONString(result));
                    } else {
                        // 请求参数
                        paramMap.put(param.getParamObjName(), result);
                    }
                }
        );


        // http请求或者 https请求
        if (interfaceTypeCode.startsWith("http")) {
            String url = interfaceTypeCode + "://" +
                    sysInterface.getInterfaceIp() + ":" +
                    sysInterface.getInterfacePort() +
                    sysInterface.getInterfaceUrl();

            Map<String,Object> allMap = new HashMap<>(headersMap);
            allMap.putAll(paramMap);
            StringSubstitutor sub  = new StringSubstitutor(allMap);
            // 替换url中的变量
            url = sub.replace(url);

            // 请求体模板和完整请求体
            String reqBodyTemplate = sysInterface.getReqBodyTemplate();
            String jsonString = "";
            // 替换请求报文模板中的变量
            if (reqBodyTemplate != null && !reqBodyTemplate.trim().isEmpty()) {
                jsonString = TemplateUtil.execute("interface-temp-bodyTemp" + getCalcCode(),reqBodyTemplate,allMap);
            }

            // 需要使用sdk发送请求的接口编号
            List<String> sdkInterfaceNo = List.of("OY15120241127004","IY15120250124001");
            // 拿到响应体
            String responseStr = null;
            String interfaceNo = sysInterface.getId();
            if (sdkInterfaceNo.contains(interfaceNo)) {
                HashMap<String,Object> allparam = new HashMap<>(paramMap);
                allparam.putAll(headersMap);
                // responseStr = sendRequestBySDKAPI(url,jsonString,allparam);
            } else {
                //  创建 HttpRequest 对象
                HttpRequest httpRequest = null;
                String reqMethod = sysInterface.getReqMethod();
                if ("get".equals(reqMethod)) {
                    httpRequest = HttpUtil.createGet(url);
                } else {
                    httpRequest = HttpUtil.createPost(url);
                }

                // 设置请求头和请求体
                httpRequest.addHeaders(headersMap);
                if (jsonString != null && !jsonString.trim().isEmpty()) {
                    httpRequest.body(jsonString);
                }

                HttpResponse httpResponse = httpRequest.execute();
                responseStr = httpResponse.body();

                InterfaceRunning interfaceRunning = null;

                String decryptionLogic = sysInterface.getDecryptionLogic();

                // 解密逻辑
                if (null != decryptionLogic && !decryptionLogic.trim().isEmpty()) {
                    interfaceRunning = SpringUtil.getBean(decryptionLogic);
                    if (interfaceRunning != null) {
                        responseStr = interfaceRunning.decode(responseStr);
                    }
                }

                // 解密后处理逻辑
                String processLogic = sysInterface.getProcessLogic();
                if (processLogic != null && !processLogic.trim().isEmpty()) {
                    interfaceRunning = SpringUtil.getBean(processLogic);
                    if (interfaceRunning != null) {
                        responseStr = interfaceRunning.afterDecode(interfaceNo,responseStr,calculateInstanceNumber);
                    }
                }

                thisResult.put(getCalcCode(),responseStr);


            }

//            HttpRequest postRequest = HttpUtil.createPost(url);
//            postRequest.addHeaders(headersMap);
//            String jsonString = JsonUtil.toJSONString(paramMap);
//            log.warn("请求参数json：{}", jsonString);
//            postRequest.body(jsonString);
//            try (HttpResponse httpResponse = postRequest.execute()) {
//                String responseStr = httpResponse.body();
//                log.debug("请求接口【{}】的结果是：{}", url, responseStr);
//                thisResult.put(getCalcCode(), responseStr);
//            }
        }
    }



    private Object getParamValue(SysInterfaceParam sysInterfaceParam, Map<String, Object> context) {
        // 参数获取类型
        String paramRetrievalTypeCode = sysInterfaceParam.getParamRetrievalTypeCode();

        // 参数获取内容
        String paramRetrievalTypeCont = sysInterfaceParam.getParamRetrievalTypeCont();

        Object result = null;

        JdbcTemplate jdbcTemplate = JdbcUtils.getJdbcTemplate();

        // 获取参数值
        switch (paramRetrievalTypeCode) {
            // 固定值
            case "fixed":
                result = paramRetrievalTypeCont;
                break;
            // java
            //   获取参数内容
            case "java":
                try {
                    result = InvokeUtil.invokeMethod(paramRetrievalTypeCont);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
                break;
            // SQL-Map
            case "sqlmap":
                String sqlMap = TemplateUtil.execute("interfaceParam-temp-sqlmap-" + getCalcCode(), paramRetrievalTypeCont, context);
                log.warn("sql语句为：{}",sqlMap);
                result = jdbcTemplate.queryForMap(sqlMap);
                break;
            // SQL-List
            case "sqllist":
                String sqlList = TemplateUtil.execute("interfaceParam-temp-sqllist-" + getCalcCode(), paramRetrievalTypeCont, context);
                log.warn("sql语句为：{}",sqlList);
                result = jdbcTemplate.queryForList(sqlList);
                break;
            // SQL-String
            case "sqlstring":
                String sqlString = TemplateUtil.execute("interfaceParam-temp-sqlstring-" + getCalcCode(), paramRetrievalTypeCont, context);
                log.warn("sql语句为：{}",sqlString);
                result = jdbcTemplate.queryForObject(sqlString, Object.class);
                break;
        }


        String paramTypeCode = sysInterfaceParam.getParamTypeCode();
        if ("string".equals(paramTypeCode)) {
            if (result instanceof String) {

            } else {
                result = JsonUtil.toJSONString(result);
            }
        } else if ("number".equals(paramTypeCode)) {
            if (result instanceof Number) {
                if (result instanceof Integer) {
                    result = ((Number) result).intValue();
                } else if (result instanceof Long) {
                    result = ((Number) result).longValue();
                } else if (result instanceof Float) {
                    result = ((Number) result).floatValue();
                } else if (result instanceof Double) {
                    result = ((Number) result).doubleValue();
                } else if (result instanceof Short) {
                    result = ((Number) result).shortValue();
                } else if (result instanceof Byte) {
                    result = ((Number) result).byteValue();
                } else {
                    throw new RuntimeException("数据类型不匹配：计算机过不为'数字'");
                }

            } else {
                throw new RuntimeException("数据类型不匹配：计算机过不为'数字'");
            }
        } else {
            throw new CalcException("返回值类型有无，目前仅有 数字和字符串");
        }
        return result;
    }


    /**
     *
     * @param url
     * @param bodyStr
     * @param allParam
     * @return
     */
//    private String sendRequestBySDKAPI (String url, String bodyStr, HashMap<String,Object> allParam) {
//        Object publicKey = allParam.get("publicKey");
//        if (publicKey == null) {
//            throw new RuntimeException("publicKey 不能为空");
//        }
//        Object systemCode = allParam.get("systemCode");
//        if (systemCode == null) {
//            throw new RuntimeException("systemCode 不能为空");
//        }
//
//        // 构建发送请求客户端
//        ApiClient apiClient = ApiClient.builder().apiUrl(url)
//                .publicKey(publicKey)
//                .systemCode(systemCode)
//                .build();
//
//        // 构建json反序列化为MyApiRequest 对象
//        MyApiRequest apiRequest = JSONUtil.toBean(bodyStr,MyApiRequest.class);
//        String jsonPrettyStr = JSONUtil.toJsonPrettyStr(apiRequest);
//        // 相应信息
//        ApiResponse<JSONObject> response = apiRequest.excute(apiClient);
//        String jsonStr = JSONUtil.toJsonStr(response);
//        return jsonStr;
//    }
//
////    private static class MyApiRequest extends ApiRequest<HashMap<String,Object>, JSONObject> {
//        public MyApiRequest() {
//            super.setResponseDataClass(JSONObject.class);
//        }
//    }

}
